
---
# Haptic 4 click

> Haptic 4 Click is a compact add-on board that enables precise haptic feedback in various electronic projects. This board features the DA7280, a haptic driver designed to drive linear resonant actuator (LRA) and eccentric rotating mass (ERM) actuators from Renesas. The DA7280 boasts a differential output drive and continuous motion sensing for calibration-free operation, coupled with wide-band support to leverage the capabilities of modern LRAs. It supports six independent haptic sequences activated directly via mikroBUS™ pins or externally through I2C or PWM signals, offering extensive flexibility for haptic configuration. Ideal for enhancing user experience in wearables, electronic peripherals, automotive interfaces, industrial controls, and AR/VR controllers, this Click board™ opens new possibilities for interactive and tactile-responsive technology.

<p align="center">
  <img src="https://download.mikroe.com/images/click_for_ide/haptic4_click.png" height=300px>
</p>

[click Product page](https://www.mikroe.com/haptic-4-click)

---


#### Click library

- **Author**        : Stefan Ilic
- **Date**          : Dec 2023.
- **Type**          : I2C type


# Software Support

We provide a library for the Haptic 4 Click
as well as a demo application (example), developed using MikroElektronika
[compilers](https://www.mikroe.com/necto-studio).
The demo can run on all the main MikroElektronika [development boards](https://www.mikroe.com/development-boards).

Package can be downloaded/installed directly from *NECTO Studio Package Manager*(recommended way), downloaded from our [LibStock&trade;](https://libstock.mikroe.com) or found on [Mikroe github account](https://github.com/MikroElektronika/mikrosdk_click_v2/tree/master/clicks).

## Library Description

> This library contains API for Haptic 4 Click driver.

#### Standard key functions :

- `haptic4_cfg_setup` Config Object Initialization function.
```c
void haptic4_cfg_setup ( haptic4_cfg_t *cfg );
```

- `haptic4_init` Initialization function.
```c
err_t haptic4_init ( haptic4_t *ctx, haptic4_cfg_t *cfg );
```

- `haptic4_default_cfg` Click Default Configuration function.
```c
err_t haptic4_default_cfg ( haptic4_t *ctx );
```

#### Example key functions :

- `haptic4_check_communication` This function checks the communication by reading and verifying the chip ID.
```c
err_t haptic4_check_communication ( haptic4_t *ctx );
```

- `haptic4_set_vibration_level` This function sets the motor vibration level.
```c
err_t haptic4_set_vibration_level ( haptic4_t *ctx, float level );
```

- `haptic4_get_vibration_level` This function reads the motor vibration level.
```c
err_t haptic4_get_vibration_level ( haptic4_t *ctx, float *level );
```

## Example Description

> This example demonstrates the use of Haptic 4 click board by controlling
  the attached motor vibration level.

**The demo application is composed of two sections :**

### Application Init

> Initializes the driver and performs the click default configuration.

```c

void application_init ( void ) 
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    haptic4_cfg_t haptic4_cfg;  /**< Click config object. */

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.
    haptic4_cfg_setup( &haptic4_cfg );
    HAPTIC4_MAP_MIKROBUS( haptic4_cfg, MIKROBUS_1 );
    if ( I2C_MASTER_ERROR == haptic4_init( &haptic4, &haptic4_cfg ) ) 
    {
        log_error( &logger, " Communication init." );
        for ( ; ; );
    }
    
    if ( HAPTIC4_ERROR == haptic4_default_cfg ( &haptic4 ) )
    {
        log_error( &logger, " Default configuration." );
        for ( ; ; );
    }
    
    log_info( &logger, " Application Task " );
}

```

### Application Task

> Changes the motor vibration level every 2 seconds between MAX and MIN, 
  and displays the currently set level on the USB UART.

```c
void application_task ( void ) 
{
    float vibration_level;
    if ( HAPTIC4_OK == haptic4_set_vibration_level ( &haptic4, HAPTIC4_VIBRATION_LEVEL_MAX ) )
    {
        if ( HAPTIC4_OK == haptic4_get_vibration_level ( &haptic4, &vibration_level ) )
        {
            log_printf( &logger, " Vibration level: %.3f \r\n\n", vibration_level );
        }
    }
    Delay_ms ( 1000 );
    Delay_ms ( 1000 );
    
    if ( HAPTIC4_OK == haptic4_set_vibration_level ( &haptic4, HAPTIC4_VIBRATION_LEVEL_MIN ) )
    {
        if ( HAPTIC4_OK == haptic4_get_vibration_level ( &haptic4, &vibration_level ) )
        {
            log_printf( &logger, " Vibration level: %.3f \r\n\n", vibration_level );
        }
    }
    Delay_ms ( 1000 );
    Delay_ms ( 1000 );
}
```


The full application code, and ready to use projects can be installed directly from *NECTO Studio Package Manager*(recommended way), downloaded from our [LibStock&trade;](https://libstock.mikroe.com) or found on [Mikroe github account](https://github.com/MikroElektronika/mikrosdk_click_v2/tree/master/clicks).

**Other Mikroe Libraries used in the example:**

- MikroSDK.Board
- MikroSDK.Log
- Click.Haptic4

**Additional notes and informations**

Depending on the development board you are using, you may need
[USB UART click](https://www.mikroe.com/usb-uart-click),
[USB UART 2 Click](https://www.mikroe.com/usb-uart-2-click) or
[RS232 Click](https://www.mikroe.com/rs232-click) to connect to your PC, for
development systems with no UART to USB interface available on the board. UART
terminal is available in all MikroElektronika
[compilers](https://shop.mikroe.com/compilers).

---
